package de.hdm_stuttgart.mi.sd1.task1;


/**
 * <p>Computing results from a single digit value.</p>
 */
public class _2_DigitExpressions {
    /**
     * <p>Computing an expression from a single digit value.</p>
     *
     * <p>A single digit value <code>d</code> may be 0, 1, 2, 3, 4, 5, 6, 7, 8 or 9. Starting from an arbitrary digit
     * value \( d \) this method computes \(d + d^2 + d^3 \). Example:</p>
     *
     * <p>A digit value of 2 will return \(2 + 2 \cdot 2 + 2 \cdot 3 \cdot 3 = 14 \).</p>
     *
     * @param digit A single digit value ranging from 0 to 9.
     * @return \( \texttt{digit} + \texttt{digit}^2 + \texttt{digit}^3 \).
     */
    static public int getValueThreeDigits(final byte digit) {
        return digit * (1 + digit * (1 + digit)); // Horner's method, https://en.wikipedia.org/wiki/Horner%27s_method
    }

    /**
     * <p>Computing an expression from a single digit up to an imposed limit.</p>
     *
     * <p>This method is a generalization of {@link #getValueThreeDigits(byte)}. Instead of imposing a fixed limit
     * for of 3 the highest product's exponent is being provided by an extra argument. Examples:</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Computing powers of a given value up to a provided exponent.</caption>
     *     <tbody>
     *         <tr>
     *             <th>digit</th>
     *             <th style="max-width: 5em;">Highest exponent</th>
     *             <th>Result</th>
     *         </tr>
     *         <tr>
     *             <td>5</td>
     *             <td style="color:red;">2</td>
     *             <td>\( 5 + 5^\color{red}{2} = 5 + 25 = 30 \)</td>
     *         </tr>
     *         <tr>
     *             <td>2</td>
     *             <td style="color:red;">4</td>
     *             <td> \( 2 + 2^2 + 2^3 + 2^\color{red}{4} = 2 + 4 + 8 + 16 = 30 \)</td>
     *         </tr>
     *
     *     </tbody>
     *
     * </table>
     *
     * <section class="implementationHints">
     *    <h4 class="implementationHints">Implementation hints:</h4>
     *    <p>Be careful with respect to arithmetic overflow errors.</p>
     * </section>
     *
     * @param digit A single digit value ranging from 0 to 9.
     * @param highestExponent The highest power to be present. Must be greater than 0.
     * @return \( \texttt{digit} + \texttt{digit}^2 + \cdots +  \texttt{digit}^{\texttt{maxLength}} \)
     */
    static public long getValue(final byte digit, final byte highestExponent) {
        long product = digit;
        long result = digit;

        for (int i = 1; i < highestExponent; i++) {
            product *= digit;
            result += product;
        }
        return result;
    }

    private _2_DigitExpressions(){/* Ignore me: Suppressing default constructor javadoc generation:
                                    javadoc does not include private constructors by default */}
}
