package de.hdm_stuttgart.mi.sd1.task1;

import java.util.Arrays;

/**
 * <p>Filtering an array of strings for containing a given substring.</p>
 */
public class _4_Filter {

    /**
     * <p>Filter a given array of strings for containing a given substring.</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Examples</caption>
     *     <tbody>
     *         <tr>
     *             <th>Values</th>
     *             <th>Filter</th>
     *             <th>Result</th>
     *             <th>Remark</th>
     *         </tr>
     *         <tr>
     *             <td>{"C<em class='myRed'>ool</em> staff here", "No staff on offer", "Bad staff"}</td>
     *             <td>"<em class='myRed'>ool</em>"</td>
     *             <td>{"C<em class='myRed'>ool</em> staff here"}</td>
     *             <td>One matching input value</td>
     *         </tr>
     *
     *         <tr>
     *             <td>{"Cool <em class='myRed'>staff</em> here",
     *             "No <em class='myRed'>staff</em> on offer", "Bad <em class='myRed'>staff</em>"}</td>
     *             <td>"<em class='myRed'>staff</em>"</td>
     *             <td>{"Cool <em class='myRed'>staff</em> here",
     *             "No <em class='myRed'>staff</em> on offer", "Bad <em class='myRed'>staff</em>"}</td>
     *             <td>All input values match</td>
     *         </tr>
     *
     *         <tr>
     *             <td>{"No luck", "Nice try!"}</td>
     *             <td>"<em class='myRed'>test</em>"</td>
     *             <td>{}</td>
     *             <td>No matching input value</td>
     *         </tr>
     *
     *     </tbody>
     *
     * </table>
     *
     * @param values An array of strings
     * @param filter A string intended to filter the given array of values.
     * @return An array of just those values containing the filter value as a substring.
     *
     */
    static public String[] filter(final String[] values, final String filter) {

        // Matching values will be stored here. We allocate sufficient space
        // in case all values are being matched.
        final String[] matchingValues = new String[values.length];

        // Number of values matching
        int matchCount = 0;

        // Check each value for being a match
        for (final String s: values) {
            if (s.contains(filter)) {
                // Copy matching value thereby incrementing match count
                matchingValues[matchCount++] = s;
            }
        }

        // Return only matching portion (Result may be empty)
        return Arrays.copyOf(matchingValues, matchCount);
    }

    private _4_Filter(){/* Ignore me: Suppressing default constructor javadoc generation:
                              javadoc does not include private constructors by default */}
}
